#import "HttpEx.h"

#define URL_TEST @"http://npaka.net/iphone/test.txt"
#define BTN_READ 0

//HttpExの実装
@implementation HttpEx

//テキストフィールドの生成
- (UITextField*)makeTextField:(CGRect)rect text:(NSString*)text {
    UITextField* textField=[[[UITextField alloc] init] autorelease];
    [textField setText:text];
    [textField setFrame:rect];
    [textField setReturnKeyType:UIReturnKeyDone];
    [textField setBackgroundColor:[UIColor whiteColor]];
    [textField setBorderStyle:UITextBorderStyleRoundedRect];
    [_textField setDelegate:self];
    return textField;
}

//テキストボタンの生成
- (UIButton*)makeButton:(CGRect)rect text:(NSString*)text tag:(int)tag {
    UIButton* button=[UIButton buttonWithType:UIButtonTypeRoundedRect];
    [button setFrame:rect];
    [button setTitle:text forState:UIControlStateNormal];
    [button setTag:tag];
    [button addTarget:self action:@selector(clickButton:) 
        forControlEvents:UIControlEventTouchUpInside];
    return button;
}

//データ→文字列
- (NSString*)data2str:(NSData*)data {
    return [[[NSString alloc] initWithData:data 
        encoding:NSUTF8StringEncoding] autorelease];
}

//初期化
- (void)viewDidLoad {
    [super viewDidLoad];
        
    //テキストフィールドの生成
    _textField=[[self makeTextField:CGRectMake(0,0,300,32) 
        text:@""] retain];
    [self.view addSubview:_textField];
        
    //読み込みボタンの生成
    UIButton* btnRead=[self makeButton:CGRectMake(0,50,90,40) 
        text:@"読み込み" tag:BTN_READ];
    [self.view addSubview:btnRead];

    //インジケーターの生成(4)
    _indicator=[[UIActivityIndicatorView alloc] init];
    [_indicator setFrame:CGRectMake(140,140,40,40)];
    [_indicator setActivityIndicatorViewStyle:
        UIActivityIndicatorViewStyleWhiteLarge];
    [_indicator setHidesWhenStopped:YES];
    [self.view addSubview:_indicator];
}

//メモリ解放
- (void)dealloc {
    [_textField release];
    [_indicator release];
    [super dealloc];
}

//テキストフィールドのリターン時に呼ばれる
- (BOOL)textFieldShouldReturn:(UITextField*)sender {
    [sender resignFirstResponder];
    return YES;
}

//ボタンクリック時に呼ばれる
- (IBAction)clickButton:(UIButton*)sender {
    if (sender.tag==BTN_READ) {
        //インジケーターのアニメーションの開始(4)
        [_indicator startAnimating];
        
        //グローバルキューで実行する処理を指定(3)
        dispatch_async(dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_DEFAULT,0), ^{
            //URLリクエストの生成(1)
            NSURLRequest* request=[NSURLRequest 
                requestWithURL:[NSURL URLWithString:URL_TEST]
                cachePolicy:NSURLRequestUseProtocolCachePolicy 
                timeoutInterval:30.0];                    
            
            //HTTP通信(2)
            NSError* error=nil;
            NSData* data=[NSURLConnection sendSynchronousRequest:request 
                returningResponse:nil error:&error];
            
            //メインキューで実行する処理を指定(3)
            dispatch_async(dispatch_get_main_queue(), ^{
                //UIの更新
                if (error==nil) {
                    _textField.text=[self data2str:data];
                } else {
                    _textField.text=@"通信エラー";
                }
                //インジケーターのアニメーションの停止(4)
                [_indicator stopAnimating];
            });
        });
    }
}

//画面を端末の向きにあわせて回転
- (BOOL)shouldAutorotateToInterfaceOrientation:
    (UIInterfaceOrientation)orientation {
    return YES;
}
@end