#import "ActionView.h"

//ActionViewの実装
@implementation ActionView

//乱数の取得
- (int)rand:(int)num {
    return abs(arc4random())%num;
}

//数値→文字列
- (NSString*)num2str:(int)num len:(int)len {
    return [NSString stringWithFormat:@"%d",num];
}

//コーダーの初期化
- (id)initWithCoder:(NSCoder*)coder {
    self=[super initWithCoder:coder];
    if (self) {
        //サイズ定数
        CGRect bounds=[[UIScreen mainScreen] bounds];
        if (bounds.size.width<bounds.size.height) {
            WIDTH =bounds.size.height;
            HEIGHT=bounds.size.width;
        } else {
            WIDTH =bounds.size.width;
            HEIGHT=bounds.size.height;
        }
        SCALE=1;
        if (WIDTH>640) {
            SCALE=2;
            WIDTH/=2;
            HEIGHT/=2;
        }
        
        //システム
        _g=[[Graphics alloc] init];
        _bmp=[[NSMutableArray array] retain];
        for (int i=0;i<4;i++) {
            [_bmp addObject:[UIImage imageNamed:
                [NSString stringWithFormat:@"act%d.png",i]]];
        }
        
        //タイマーの開始
        _init=S_TITLE;
        [NSTimer scheduledTimerWithTimeInterval:0.1f
            target:self selector:@selector(onTick:)
            userInfo:nil repeats:YES];        
    }
    return self;
}

//メモリ解放
- (void)dealloc {
    [_g release];
    [_bmp release];
    [super dealloc];
}

//描画
- (void)drawRect:(CGRect)rect {
    //コンテキストの指定
    [_g setContext:UIGraphicsGetCurrentContext()];
    [_g setScaleW:SCALE h:SCALE];
    
    //バッファのクリア
    [_g setColorR:0 g:0 b:0];
    [_g fillRectX:0 y:0 w:WIDTH h:HEIGHT];
    
    //シーンの初期化
    if (_init>=0) {
        _scene=_init;
        //タイトル
        if (_scene==S_TITLE) {
            _message =@"Touch Start";
            _score   =0;
            _playerY =HEIGHT-50;
            _jumpPow =0;
            _jumpAble=YES;
            _mapDX   =0;
            for (int i=0;i<10;i++) _mapH[i]=1;
        }
        //プレイ
        else if (_scene==S_PLAY) {
            _message  =nil;
            _touchDown=NO;
        } 
        //ゲームオーバー
        else if (_scene==S_GAMEOVER) {
            _message=@"Game Over";
        }
        _init=-1;
        _tick=0;
        _key =KEY_NONE;
    }    
    
    //プレイ時の処理
    if (_scene==S_PLAY) {
        //スコア加算
        _score++;
        
        //衝突判定
        if (_playerY>HEIGHT-_mapH[2]*50) {
            _init=S_GAMEOVER;
            _jumpAble=NO;
        }
        //上移動
        else if (_jumpPow>=0) {
            _playerY-=_jumpPow*2;
            _jumpPow--;
        }
        //下移動(3)
        else {
            _playerY+=12;
            _jumpAble=NO;
            if (_mapH[2]!=0 &&
                _playerY>HEIGHT-_mapH[2]*50) {
                _playerY=HEIGHT-_mapH[2]*50;
                _jumpAble=YES;
            }
        }
        //スクロール処理(2)
        _mapDX-=10;
        if (_mapDX==-50) {
            _mapDX=0;
            
            //地面の高さを左にシフト
            for (int i=0;i<11;i++) _mapH[i]=_mapH[i+1];
            
            int idx=[self rand:6];
            //[10]が穴の時は[9]と同じ高さ
            if (_mapH[10]==0) {
                _mapH[11]=_mapH[7];
            } 
            //)1/6の確率で[11]を穴
            else if (idx==0) {
                _mapH[11]=0;
            } 
            //1/6の確率で[11]を[10]の高さ+1
            else if (idx==1) {
                _mapH[11]=_mapH[10]+1;
                if (_mapH[11]>4) _mapH[11]=4;
            } 
            //1/6の確率で[11]を[10]の高さ-1
            else if (idx==2) {
                _mapH[11]=_mapH[10]-1;
                if (_mapH[11]<1) _mapH[11]=1;
            }
        }
        
        //ジャンプ処理(3)
        if (_jumpAble) {
            if (_touchDown) {
                _jumpAble=NO;
                _jumpPow =14;
            }
        } else {
            if (!_touchDown) {
                _jumpPow=-10;
            }
        }
    }
    //ゲームオーバー時の処理
    else if (_scene==S_GAMEOVER) {
        if (_playerY<700) _playerY+=16;
    }
    
    //背景の描画
    [_g drawImage:[_bmp objectAtIndex:0] x:0 y:0 w:WIDTH h:HEIGHT];
    
    //プレイヤーの描画
    int idx=3;
    if (_jumpAble) idx=2+_score%2;
    [_g drawImage:[_bmp objectAtIndex:idx] x:75 y:_playerY-50];
    
    //地面の描画
    for (int i=0;i<12;i++) {
        [_g drawImage:[_bmp objectAtIndex:1] x:_mapDX+i*50 y:HEIGHT-_mapH[i]*50];
    }
    
    //スコアの描画
    [_g setColorR:1 g:1 b:1];
    [_g setFont:[UIFont systemFontOfSize:24]];
    [_g drawString:[NSString stringWithFormat:@"SCORE %@",
        [self num2str:_score len:6]] x:10 y:30];
    
    //メッセージの描画
    if (_message!=nil) {
        [_g drawString:_message x:(WIDTH-[_g stringWidth:_message])/2 y:50];
    }
    
    //時間経過
    _key=-999;
    _tick++;
    if (_tick>999) _tick=100;
}

//定期処理
- (void)onTick:(NSTimer*)timer {
    [self setNeedsDisplay];
}

//タッチ開始イベントの処理
- (void)touchesBegan:(NSSet*)touches withEvent:(UIEvent*)event {
    if (_scene==S_TITLE) {
        _init=S_PLAY;
    } else if (_scene==S_PLAY) {
        _touchDown=YES;
    } else {
        if (_playerY>=HEIGHT+50) _init=S_TITLE;
    }
}

//タッチ終了イベントの処理
- (void)touchesEnded:(NSSet*)touches withEvent:(UIEvent*)event {
    if (_scene==S_PLAY) {
        _touchDown=false;
    }
}

//タッチキャンセルイベントの処理
- (void)touchesCancelled:(NSSet*)touches withEvent:(UIEvent*)event {
    [self touchesEnded:touches withEvent:event];
}
@end
